@@ -46,9 +46,11 @@
 #' @param check.path path to \sQuote{check} directory, where check files are
 #' created. The default (NULL) will not produce any check files.
 #' @param failflow_fn filename for duplicate flow data with rate zero
+#' @param failnounit_fn filename for MAR data with missing unit
 #' @param failunit_fn filename for MAR data with invalid unit
 #' @param failnowgt_fn filename for infusion data with missing weight where unit
 #' indicates weight is required
+#' @param censor_date_fn filename containing censor times created with invalid dose data
 #' @param infusion.unit acceptable unit for infusion data
 #' @param bolus.unit acceptable unit for bolus data
 #' @param bol.rate.thresh upper limit for bolus rate; values above this are invalid
@@ -63,24 +65,21 @@
 #' @return structured data set
 #'
 #' @examples 
-#' \dontrun{
 #' # flow data for 'Fakedrug1'
 #' flow <- data.frame(mod_id=c(1,1,2,2,2),
 #'                    mod_id_visit=c(46723,46723,84935,84935,84935),
-#'                    record.date=c("7/5/2019 5:25","7/5/2019 6:01",
-#'                                  "9/4/2020 3:21", "9/4/2020 4:39",
-#'                                  "9/4/2020 5:32"),
+#'                    record.date=c("07/05/2019 5:25","07/05/2019 6:01",
+#'                                  "09/04/2020 3:21", "09/04/2020 4:39",
+#'                                  "09/04/2020 5:32"),
 #'                    Final.Weight=c(6.75,6.75,4.5,4.5,4.5),
 #'                    Final.Rate=c(rep("1 mcg/kg/hr",2),
 #'                                 rep("0.5 mcg/kg/hr",3)),
 #'                    Final.Units=c("3.375","6.5",
 #'                                  "2.25","2.25","2.25"))
-#' flow[,'Perform.Date'] <- pkdata::parse_dates(EHR:::fixDates(flow[,'record.date']))
+#' flow[,'Perform.Date'] <- pkdata::parse_dates(flow[,'record.date'])
 #' flow[,'unit'] <- sub('.*[ ]', '', flow[,'Final.Rate'])
 #' flow[,'rate'] <- as.numeric(sub('([0-9.]+).*', '\\1', flow[,'Final.Rate']))
 #'
-#' saveRDS(flow, 'flow.rds')
-#'
 #' # mar data for 4 fake drugs
 #' mar <- data.frame(mod_id=rep(1,5),
 #'                   Date=rep("2019-07-05",5),
@@ -92,26 +91,20 @@
 #'                   `med:route`=rep("IV",5),
 #'                   `med:given`=rep("Given",5),
 #'                   check.names=FALSE)
-#'                   
-#' saveRDS(mar, 'mar.rds')
 #' 
 #' # medcheck file for drug of interest ('Fakedrug1')
 #' medcheck <- data.frame(medname="Fakedrug1",freq=4672)
 #' 
-#' write.csv(medcheck, 'medcheck.csv')
-#' 
-#' 
-#' run_MedStrI(mar.path='mar.rds',
+#' run_MedStrI(mar.path = mar,
 #'             mar.columns = list(id = 'mod_id', datetime = c('Date','Time'),
 #'                                dose = 'med:dosage', drug = 'med:mDrug', given = 'med:given'),
-#'             flow.path='flow.rds',
+#'             flow.path = flow,
 #'             flow.columns = list(id = 'mod_id', datetime = 'Perform.Date',
 #'                                 finalunits = 'Final.Units', unit = 'unit',
 #'                                 rate = 'rate', weight = 'Final.Weight'),
-#'             medchk.path='medcheck.csv',
-#'             check.path=tempdir(),
-#'             drugname='fakedrg1')
-#'}
+#'             medchk.path = medcheck,
+#'             check.path = tempdir(),
+#'             drugname = 'fakedrg1')
 #'
 #' @export
 
@@ -127,8 +120,10 @@ run_MedStrI <- function(mar.path,
                         wgt.columns = list(),
                         check.path = NULL,
                         failflow_fn = 'FailFlow',
+                        failnounit_fn = 'NoUnit',
                         failunit_fn = 'Unit',
                         failnowgt_fn = 'NoWgt',
+                        censor_date_fn = 'CensorTime',
                         infusion.unit = 'mcg/kg/hr',
                         bolus.unit = 'mcg',
                         bol.rate.thresh = Inf,
@@ -175,6 +170,8 @@ run_MedStrI <- function(mar.path,
   mar.weightCol <- mar.col$weight
   mar.givenCol <- mar.col$given
 
+  censor_opts <- vector('list', 3)
+
   if(!is.null(mar.drugCol) && !is.null(medchk.path)) {
     ## medChecked data
     list.med <- read(medchk.path)[['medname']]
@@ -194,29 +191,52 @@ run_MedStrI <- function(mar.path,
   }
   rm(medMAR)
 
-  if(length(mar.doseCol) == 2) {
-    rate <- dm[,mar.doseCol[1]]
-    unit <- dm[,mar.doseCol[2]]
-  } else {
-    rate <- sub('([0-9.]+).*', '\\1', dm[,mar.doseCol])
-    unit <- sub('.*[ ]', '', dm[,mar.doseCol])
-  }
-  dm[,'unit'] <- unit
-  dm[,'rate'] <- suppressWarnings(as.numeric(rate))
-  if(length(mar.datetimeCol) == 2) {
-    marDT <- paste(dm[,mar.datetimeCol[1]], dm[,mar.datetimeCol[2]])
-  } else {
-    marDT <- dm[,mar.datetimeCol]
-  }
-  dm[,'date.time'] <- pkdata::parse_dates(marDT)
-  # rename "weight" column if necessary
-  if(!is.null(mar.weightCol) && mar.weightCol != 'weight') {
-    names(dm)[names(dm) == mar.weightCol] <- 'weight'
-    mar.weightCol <- 'weight'
-  }
-  hasUnit <- !is.na(unit)
-  inf0 <- dm[hasUnit & unit == infusion.unit,]
+  dm <- setDoseMar(dm, mar.doseCol, mar.datetimeCol, mar.weightCol)
   reqInfusionColumns <- c(mar.idCol, 'date.time', 'unit', 'rate', mar.weightCol)
+
+  hasUnit <- !is.na(dm[,'unit'])
+  # handle missing unit
+  if(!is.null(check.path)) {
+    dmnu <- dm[!hasUnit,]
+    if(nrow(dmnu) > 0) {
+      nounitfn <- file.path(check.path, paste0('fail', failnounit_fn,'-', drugname, '.csv'))
+      nounitfixfn <- sub('fail', 'fix', nounitfn)
+
+      mf <- cbind(dmnu[,reqInfusionColumns], dmnu[,mar.drugCol], flag = 'exclude')
+      msg <- sprintf('%s rows with no unit, see file %s AND create %s', nrow(mf), nounitfn, nounitfixfn)
+      writeCheckData(mf, nounitfn, msg)
+
+      if(file.access(nounitfixfn, 4) != -1) {
+        hasfix <- read.csv(nounitfixfn, stringsAsFactors = FALSE)
+        if(nrow(dmnu) != nrow(hasfix)) {
+          wmsg <- sprintf('all rows in fix file %s should match respective rows in fail file %s\n', nounitfn, nounitfixfn)
+          stop(wmsg)
+        }
+        toKeep <- hasfix[,'flag'] != 'exclude'
+        nFixed <- sum(toKeep)
+        nBad <- sum(!toKeep)
+        if(nFixed > 0L) {
+          # update required columns (ideally just "unit")
+          dmnu[toKeep, reqInfusionColumns] <- hasfix[toKeep, reqInfusionColumns]
+          message(sprintf('file %s read, %s records added', nounitfixfn, nFixed))
+          dm0 <- setDoseMar(dmnu[toKeep,], mar.doseCol, 'date.time', mar.weightCol)
+          dm1 <- rbind(dm[hasUnit,], dm0)
+          # re-order data
+          dm <- dm1[order(dm1[,mar.idCol], dm1[,'date.time']),]
+          hasUnit <- !is.na(dm[,'unit'])
+        }
+        ## unfixed/bad data should be censored
+        if(nBad > 0L) {
+          censor_opts[[1]] <- dmnu[!toKeep, c(mar.idCol, 'date.time')]
+        }
+      } else {
+        censor_opts[[1]] <- dmnu[, c(mar.idCol, 'date.time')]
+      }
+    }
+  }
+
+  unit <- dm[,'unit']
+  inf0 <- dm[hasUnit & unit == infusion.unit,]
   inf1 <- inf0[,reqInfusionColumns]
 
   bol <- dm[hasUnit & unit == bolus.unit,]
@@ -308,21 +328,28 @@ run_MedStrI <- function(mar.path,
 
     coi <- c(mar.idCol, 'date.time', mar.drugCol, 'rate', 'unit', 'weight', 'weight.date.time')
     mf <- cbind(dm[,coi], flag = 'exclude')
-    msg <- sprintf('%s rows with units other than %s or %s, see file %s AND create %s\n',
+    msg <- sprintf('%s rows with units other than %s or %s, see file %s AND create %s',
                 nrow(mf),infusion.unit, bolus.unit, unitfn, unitfixfn)
     writeCheckData(mf, unitfn, msg)
 
     if(file.access(unitfixfn, 4) != -1) {
       hasfix <- read.csv(unitfixfn, stringsAsFactors = FALSE)
-      hasfix <- hasfix[hasfix[,'flag'] == 'keep', reqInfusionColumns]
-      if(nrow(hasfix)) {
-        inf1 <- rbind(inf0[,reqInfusionColumns], hasfix)
-        cat(sprintf('file %s read, %s records added\n', unitfixfn, nrow(hasfix)))
+      toKeep <- hasfix[,'flag'] != 'exclude'
+      nFixed <- sum(toKeep)
+      nBad <- sum(!toKeep)
+      if(nFixed > 0L) {
+        inf1 <- rbind(inf0[,reqInfusionColumns], hasfix[toKeep, reqInfusionColumns])
+        message(sprintf('file %s read, %s records added', unitfixfn, nFixed))
       }
+      ## unfixed/bad data should be censored
+      if(nBad > 0L) {
+        censor_opts[[2]] <- hasfix[!toKeep, c(mar.idCol, 'date.time')]
+      }
+    } else {
+      censor_opts[[2]] <- mf[, c(mar.idCol, 'date.time')]
     }
   } else {
-    cat(sprintf('no units other than %s or %s, file %s not created\n',
-                infusion.unit, bolus.unit, unitfn))
+    message(sprintf('no units other than %s or %s, file %s not created', infusion.unit, bolus.unit, unitfn))
   }
 
   # infusionData_mod() accepts MAR data (inf1) with units
@@ -358,21 +385,39 @@ run_MedStrI <- function(mar.path,
   if(!is.null(check.path) && length(rnums)) {
     needfix <- inf[rnums,]
     nofix <- inf[-rnums,]
+    if('orig_rate' %in% names(inf)) {
+      # preserve original rate
+      needfix[,'rate'] <- needfix[,'orig_rate']
+      needfix[,'orig_rate'] <- NULL
+      nofix[,'orig_rate'] <- NULL
+    }
     nowgtfn <- file.path(check.path, paste0('fail', failnowgt_fn,'-', drugname, '.csv'))
     nowgtfixfn <- sub('fail', 'fix', nowgtfn)
-    msg <- sprintf('%s rows from %s subjects with "%s" in infusion unit but missing weight, see file %s AND create %s\n',
+    msg <- sprintf('%s rows from %s subjects with "%s" in infusion unit but missing weight, see file %s AND create %s',
                 nrow(needfix), length(unique(needfix[,mar.idCol])), weightunit, nowgtfn, nowgtfixfn)
     writeCheckData(needfix, nowgtfn, msg)
 
     if(file.access(nowgtfixfn, 4) != -1) {
       hasfix <- read.csv(nowgtfixfn, stringsAsFactors = FALSE)
-      if(nrow(hasfix)) {
+      isFixed <- which(!is.na(hasfix[,'rate']) & !is.na(hasfix[,'weight']))
+      if(length(isFixed)) {
+        # multiply hourly rate per weight unit by weight to get hourly rate
+        # note: this is awkward, would be better to provide data in `missing.wgt.path`
+        # rather than a fix file
+        hasfix[,'rate'] <- hasfix[,'rate'] * hasfix[,'weight']
         inf <- rbind(nofix, hasfix[,names(nofix)])
         inf <- inf[order(inf[,mar.idCol], inf[,'date.time']),]
-        cat(sprintf('file %s read, %s records corrected\n', nowgtfixfn, nrow(hasfix)))
+        message(sprintf('file %s read, %s records corrected', nowgtfixfn, nrow(hasfix)))
       }
+    } else {
+      # no fix file
+      censor_opts[[3]] <- needfix[, c(mar.idCol, 'date.time')]
     }
   }
+  # original rate may have been preserved for fix file - delete it now
+  if('orig_rate' %in% names(inf)) {
+    inf[,'orig_rate'] <- NULL
+  }
 
   # combine infusion and bolus
   hourly <- merge_inf_bolus(inf, bol[,c(mar.idCol,'date.time','rate')])
@@ -384,6 +429,24 @@ run_MedStrI <- function(mar.path,
     names(hourly)[1] <- mar.idCol
   }
 
+  # make sure times are POSIXct
+  for(i in seq_along(censor_opts)) {
+    if(!is.null(censor_opts[[i]])) {
+      censor_opts[[i]][,'date.time'] <- pkdata::parse_dates(censor_opts[[i]][,'date.time'])
+    }
+  }
+  censor_opts <- do.call(rbind, censor_opts)
+  if(is.null(check.path)) {
+    warning('no check.path is provided; censor dates will not be utilized')
+  } else if(!is.null(censor_opts)) {
+    # find "min" date-time for mod_id
+    censor_opts <- censor_opts[order(censor_opts[,mar.idCol], censor_opts[,'date.time']),]
+    censor_date <- censor_opts[!duplicated(censor_opts[,mar.idCol]),]
+    cdfn <- file.path(check.path, paste0(censor_date_fn,'-', drugname, '.csv'))
+    msg <- sprintf('censor dates created, please see %s', cdfn)
+    writeCheckData(censor_date, cdfn, msg)
+  }
+
   cdArgs <- list(
     doseData = hourly, idVar = mar.idCol, dateVar = "date.dose",
     otherDoseTimeVar = NULL, otherDoseVar = NULL, otherVars = c('given.dose','maxint','weight')
